@ This GAUSS code contains procs used for forecasting ACH model @

proc(3)=hazy(dur,eve,zach,procstart);
    @ This proc evaluates calculates vector of hazards for the ach model 
inputs:
	dur = (tproc x 1) vector of lagged durations; row t is length of most recent duration
		completed as of t
	eve = (tproc x 1) vector, = 1 if target change occurred on date t
	zach = (tproc x kach) matrix of explanatory variables used in ACH
	procstart = calendar date of first element of (tproc x 1) vectors, e.g., procstart =2
		 means that first element of eve is whether target change occurred on 3/8/84
outputs:
	haz = (tproc x 1) vector of hazards; row t is probability that target will change
		at date t given info at t-1 
	q = (tproc x 1) vector of values of q for each date
	lagq = scalar for the value of q as of the last previous date of a target change
global:
	meandur1 = mean duration for subsample 1
	bet1 = beta parameter for subsample 1
	thet1 = theta parameter for subsample 1
	del 1 = delta parameter for subsample 1
	ditto for 2
	tbreak = first observation of subsample 2  
	bhat = VAR coefficients used to predict spread
	vroot = cholesky decomposition (i.e., square root) of variance-covariance matrix
		of this VAR @

local  haz,tproc,q,thet,bet,del,it, lagq,lamb, mz,m;

tproc = rows(zach);
q = zeros(tproc,1);

@ generate q vector @ 
q[1,1] = meandur1*thet1/(1 - bet1);
thet = thet1;
bet = bet1;
del = del1;
lagq = q[1,1];

it = 2;
do until it > tproc;
	if it == tbreak - procstart +1;
		q[it,1] = meandur2*thet2/(1 - bet2);
		thet = thet2;
		bet = bet2;
		del = del2;
		lagq = q[it,1];
	else;
	        q[it,1] = thet*dur[it-1,1] + bet*lagq;		
		if eve[it,1] > 0;
			lagq = q[it,1];
		endif;
	endif;
it = it+1;
endo;

  @ "";"q vector is"; q'; @
 			        	
@ calculate hazard @
mz = zeros(tproc,1);
if ( (kshort == 1) or (tproc < tbreak) );
	mz = q + zach*del1;
else;
	mz[1:tbreak-procstart,1] = q[1:tbreak-procstart,1] + zach[1:tbreak-procstart,.]*del1; 
	mz[tbreak-procstart+1:tproc,1] = q[tbreak-procstart+1:tproc,1] + 
		zach[tbreak-procstart+1:tproc,.]*del2;
endif;
mz = maxc(   ( (mz .>0) .* (2*dellit*mz^2) ./ (dellit^2 + mz^2))' | (mz .* (mz .> dellit))'  ); 
mz = mz + delsmall;
haz = 1 ./ (1 +  mz );
@ "hazards are";
haz'; @
retp(haz,q,lagq);
endp;


/* ========================================================== 
	proc to calculate probabilities from ordered probit model
    ========================================================== */
proc probfore(zt,del,cvec);
@  inputs:
	zt  = (tproc x kproc) matrix of observations on variables that are used in ordered probit
		row t is observations known at t for forecasting t
	del = (kproc x 1) vector of coefficients on zt
	cvec = (kbin x 1) vector of cutoffs for each of the kbin+1 bins
    output:
	zhat = (tproc x kbin+1) vector of probabilities of each bin
		 row t is probability for date t based on data observed t-1
 @

local zhat, tproc,kbin,ihat;
tproc = rows(zt);
kbin = rows(cvec);
zhat = zt*del;     @ zhat is (tproc x 1) vector of fits for ordered probit @
zhat = cdfn(ones(tproc,1)*cvec' - zhat*ones(1,kbin)); 
		 @ zhat is now (tproc x kbin) matrix whose jth column is probability
			of latent variable falling in bin j or lower @
zhat = zeros(tproc,1) ~ zhat ~ ones(tproc,1);
		  @ fills in first and last column as appropriate @
zhat = zhat[.,2:kbin+2] - zhat[.,1:kbin+1];
		@ zhat is now (tproc x kbin+1) matrix whose jth column is probability of falling
			     in jth bin @
retp(zhat);
endp;


/* ==========================================================
		proc to calculate values target assumes under each bin
============================================================*/
proc binval(tarprev,sj);
@ inputs:
	tarprev = (tproc x 1) vector of values of target, row t is target for date t-1	
	sj = (kbin2 x 1) vector of values of change if falls in each bin
    output:
	ihat = (tproc x kbin2) matrix of values of target; row t, col j is value of target for
		date t if falls in bin j @
local ihat;
ihat =  tarprev * ones(1,rows(sj))   + ones(cols(tarprev),1)*sj'; 
retp(ihat);
endp;

/* ========================================================== 
		proc to conduct one-period-ahead forecasts from ACH model
============================================================ */
proc achfore(hz,zhat,ihat,tarprev);
@  inputs:
	hz = (tproc x 1) vector of hazards for tproc observations
		row t is probability of target change for date t
	zhat = (tproc x kbin2) matrix of probabilities of each bin
		 row t is probability for date t based on data observed t-1
	ihat = (tproc x kbin2) matrix of values of target for each bin 
	tarprev = (tproc x 1) vector of values of target, row t is target for date t-1	
    output:
	ffore = (tproc x 1) vector of forecasts
		 row t is forecast for target for date t  @

local ffore,tproc;
tproc = rows(hz);
ffore = ihat .* zhat;
ffore = sumc(ffore');   @ ffore is now (tproc x 1) vector whose tth row is expected value
			of target for date t+1 given that a change occurs at t+1 @
ffore = (  (ones(tproc,1) - hz) .* tarprev  ) + (  hz .* ffore  );
		@ ffore is now (tproc x 1) vector whose tth row is forecast of target for t+1 @
retp(ffore);
endp;

/*  ========================================================================
		proc to update forecasts of variables and simulate one period
   ========================================================================*/
proc(8)=onemore(qt,it_1,sp6t,changet_1,procdate,durt_1,duron,lagq);

@ inputs:
	qt = value of q for date t
	it_1 = value of target (perhaps result of previous simulation) for date t-1
	sp6t = value of spread (perhaps result of previous simulation) for date t
	changet_1 = value of most recent change in target observed prior to date t
	procdate = calendar date associated with t
	durt_1 = length of most recent duration completed as of date t - 1
	duron = length of ongoing duration through date t
	lagq = value of q as of most recent change through date t
   outputs:
	qnext = value of q for date t+1
	ifore = forecast value for target for date t
	isim = simulated value for target for date t
	sp6sim = simulated value of spread for date t+1
	changet = value of most recent change in target (given this simulation) that
			will have been observed prior to date t+1
	durt = length of most recent duration completed as of date t, according to simulation
   globals:
	capt = length of observed actual sample
	zach = (capt x 4) matrix of observations on variables used by ach for actual 
		complete sample
	bet1 = beta parameter for subsample 1
	thet1 = theta parameter for subsample 1
	del 1 = delta parameter for subsample 1
	ditto for 2
	tbreak = first observation of subsample 2 	
	delprob = vector of coefficients for ordered probit model
	cvec = cutoff points for ordered probit model
	sj = vector of increments in which target is presumed to be changed @
local ht, qnext, ifore,isim,sp6sim,changet,
	zacht, zprobt, datetry, probs, bins, u1, newevent, phat, u2, iq, ij, kbin2, xvarnew,
	durt, durnext;

kbin2 = rows(sj);

/* ---------------------------------------------------
	calculate zacht */

datetry = procdate;   @ this section calculates FOMC dates given their 6-period repetition @
restarted:
if datetry > capt;
	datetry = datetry - 6;
	goto restarted;
endif;
zacht = zach[datetry,.];
zacht[1,4] = abs(sp6t);
@ "zacht";;zacht; @

/* --------------------------------------------------
	calculate zprobt */

zprobt = changet_1 ~ sp6t;

/* --------------------------------------------------
	calculate ht */

@ "qt";;qt;  @
if procdate < tbreak;
	ht = qt + zacht*del1;
else;
	ht = qt + zacht*del2;
endif;
ht = maxc(   ( (ht .>0) .* (2*dellit*ht^2) ./ (dellit^2 + ht^2))' | (ht .* (ht .> dellit))'  ); 
ht = ht + delsmall;
ht = 1 ./ (1 +  ht );
@ "ht";;ht; @
/* -----------------------------------------
	calculate forecast of target */

probs = probfore(zprobt,delprob,cvec);
@ "probs";;probs; @
bins = binval(it_1,sj);
ifore = achfore(ht,probs,bins,it_1);
@ "ifore";;ifore; @
/* ------------------------------------------
	simulate value for target */

u1 = rndus(1,1,dseed);
newevent = (u1 .< ht);
phat = probs ~ 1;    @ adding column of 1's simplifies code to follow @
if newevent > 0;
	u2 = rndus(1,1,dseed);	
	@ "u2";;u2; @
	iq = 0;
	ij = 1;
	do until ij > kbin2;
		iq = iq + phat[1,ij];
		if u2 < iq;
			@ "chose bin";;ij; @
			isim = it_1 + sj[ij,1];
			ij = kbin2 + 1;
		endif;
	ij = ij + 1;
	endo;
else;
	isim = it_1;
endif;



/* ----------------------------------
	update value of change and duration */
if isim == it_1;
	changet = changet_1;
	durt = durt_1;
	durnext = duron + 1;
else;
	changet = isim - it_1;
	durt = duron;
	durnext = 1;
	lagq = qt;
endif;

/* -------------------------------------
	update value for q */
if procdate < tbreak;
        qnext = thet1*durt + bet1*lagq;		
elseif procdate > tbreak; 
	qnext = thet2*durt + bet2*lagq;
else;
	qnext = meandur2*thet2/(1 - bet2);
endif;

/* -------------------------------------
	simulate new value for spread */
xvarnew = isim ~ 1 ~ it_1 ~ sp6t;
@ "xvarnew";;xvarnew; @
u1 = rndns(1,1,dseed);
sp6sim = xvarnew*bhat' + (vroot*u1)';


retp(qnext, ifore, isim, sp6sim, changet,durt,durnext,lagq);
endp;


/*====================================================================
	proc to calculate multiperiod forecasts and simulations 
  ==================================================================== */
proc(2)=multiperiod(ihist,hor,procstart);

local hazs, durs, eves, changes, tproc, procdate, qs, qt, isims, ifores, ih, sp6t, 
	changet_1, iforet, it_1, durt_1, duron, lagq;

tproc = rows(ihist);
procdate = tproc + procstart - 1;
{durs, eves, changes, duron} = dureve(ihist);
durt_1 = durs[tproc-1,1];
{hazs, qs,lagq} = hazy(durs, eves, zach[procstart:procdate,.],procstart);
isims = zeros(1,hor);
ifores = zeros(1,hor);
qt = qs[tproc,1];
it_1 = ihist[tproc-1,1];
sp6t = spread6[procdate,1];
changet_1 = changes[tproc,1];


ih = 1;
do until ih > hor;
	 @ "----------------------";;"ih";;ih; 
	"it_1";;it_1;
	"durt_1";;durt_1;
	"sp6t";;sp6t; @
	{qt, iforet, it_1, sp6t, changet_1,durt_1,duron,lagq}
		= onemore(qt, it_1, sp6t, changet_1, procdate,durt_1,duron,lagq);
	procdate = procdate+1;
	ifores[1,ih] = iforet;
	isims[1,ih] = it_1;
ih = ih+1;
endo;

retp(ifores, isims);
endp;


/*  ========================================================================
		proc to calculate history of durations and events for given set of targets
    ========================================================================*/			 
proc(4)=dureve(targs);
	@ input:
		targs = (tproc x 1) vector of possibly counterfactual targets
	    output
		durs = (tproc x 1) vector of lagged durations; row t is length of most recent
				duration completed as of t
		eves = (tproc x 1) vector, = 1 if target change occurred on date t 
		changes = (tproc + 1 x 1) vector, row t is magnitude of most recent change 
			observed prior to week t @
local durs, eves,changes,it,durnew,durold,changeold,tproc;
tproc = rows(targs);

durs = ones(tproc,1);
eves = zeros(tproc,1);
changes = zeros(tproc,1);
durnew = 1;
durold = 1;
changeold = 0;
it = 2;
do until it > tproc;
	changes[it,1] = changeold;
	if targs[it,1] .ne targs[it-1,1];
		eves[it,1] = 1;
		changeold = targs[it,1] - targs[it-1,1];
		durs[it,1] = durnew;
		durold = durnew;
		durnew = 1;
	else;
		durs[it,1] = durold;
		durnew = durnew + 1;
	endif;
it = it+1;
endo;
if targs[tproc,1] .ne targs[tproc-1,1];
	durnew = durs[tproc,1];
endif;
retp(durs,eves,changes|changeold,durnew);
endp;






	

	




