@ This GAUSS code controls impulse-response analysis @
	output file=junk reset;

format /mb1;

/* ==================================================
	load raw monthly data
  ==================================================== */
nt = 505;
load yz[nt,7] = ..\..\newdata\varmonth\mvardat.csv;
@ first row 1959:2 last row 2001:2
	col (1)  date
	col (2)  employment
	col (3)  price
	col (4)  pcom
	col (5)  fed funds rate
	col (6) nbrx
	col (7) dm2 
	note that March, 1984 corresponds to observation # 302 of yz  
		and April, 1985 (the first set of usable forecasts) corresponds to
		observation # 315 @

yq = yz[.,2:7];    @ drops the date @

/* =======================================================
	set global variables used in procs
    ======================================================= */
nfest = 13;   @ nfest is the first observation used for estimation (after lags) @
nq = 191;      @ nq is the number of usable observations from the forecast and
			 VAR data base combined @
nuse = nq;  @ orig code: nt - nfest +1;  nuse is the number of observations used @
plag = 12;       @ plag is number of lags in VAR @
vhat = 0;         @ this will be set to the structural variance-covariance matrix for VAR @
ks = 12;            @ ks is number of horizons used for forecasts or impulse-response @
ny = 6;            @ ny is number of left-hand variables in VAR @
kc = 2;             @ kc = 2 prints output of VAR analysis, kc = 1 suppresses output @
phi = 0;             @ phi will contain lagged VAR coefficients; set to 0 to load procs @
b0 = eye(ny);    @ indicates ordinary nonstructural VAR will be used @

#include varproc2.g;   @ this adds GAUSS VAR procs @

/* =======================================================
	load monthly versions of various restricted weekly ACH forecasts
  ======================================================== */
h0 = 303;
hormonths = 12;
	@ Note that the original series run from 1985:4 (first row) to 2001:3 (row nq+1) 
		first column is date.  Only first nq are read in in order to keep same length
		as monthly data base @

load basemon[nq,hormonths+1] = ..\foreout\monthly\Base.mon;       @ base forecasts @

load it1mon[nq,hormonths+1] = ..\foreout\monthly\It1.mon;    
	   @ forecast that forced target for last week
	 	of month tau to equal target for last week of month tau - 1  @

load it1and25mon[nq,hormonths+1] = ..\foreout\monthly\It1and25.mon;
	@ forecast that forced target for last week of month tau
		to equal target for last week of month tau - 1 plus 25 basis points @

load ihatmon[nq,hormonths+1] = ..\foreout\monthly\Ihat.mon;
	@ forecast that forced target for last week of month tau
		to equal predicted target as of last week of month tau - 1  @

load lagtarmon[nq,2] = ..\foreout\monthly\lagtar.mon;
	@ series equal to the value of the fed funds target as of the end of the preceding
		month; first column is date @ 

"check: following should correspond to effect of 25-basis-point increase on
the path of fed funds rate";
junny = (it1and25mon - it1mon)/.25;
meanc(junny)';

@ junny = lagtarmon[1:nq,.] ~ basemon[1:nq,1] ~ yz[315:nt,1] ~ yz[315:nt,5];
junny[1:20,.];
end; @

/* ========================================================
	estimate VAR
========================================================== */

@ create individual vectors y1 and y2 @
y1 = yq[.,1:4];    @ y1 contains employment, price, pcom, and fed funds @
y2 = yq[.,5:6];    @ y2 contains nbrx and dm2 @

@	construct data matrix with lags of all variables, 1960:2 to 2001:2  @
xq0 = multlags(yq,plag,nfest);

xq = xq0;

@	estimate VAR coefficients for y1 @
{phihat,vhat1,ehat1} = varit(y1[nfest:nt,.],xq0);
@ note phihat is ( 4 x (1 + plag*ny) ) @

@  estimate VAR plus contemporaneous coefficients for y2 @
{dhat,vhat2,ehat2} = varit(y2[nfest:nt,.],y1[nfest:nt,.]~xq0);
@ note that dhat is ( 2 x (4 + 1 + plag*ny) ) @

/***************************************************************************
	convert VAR data base to same dates as forecast data base 
 ***************************************************************************/
xq0 = xq0[315-nfest+1:rows(xq0),.];
     @ first row of xq0 is now vector of explanatory variables for 1985:4 @
"check";
	junky = multlags(yz[.,1],plag,nfest);
	"The first row of xq0 is the month following";;junky[315-nfest+1,2];
	"rows of xq0 is";;rows(xq0);; "which should equal";;nq;

icont = 0;         @ set icont = 1 to restrict target to take on the value implied by kcont 
			     set icont = 2 to shock VAR forecast of target by 0.25 @
kcont = 0;

/***************************************************************************
	procs to do basic forecasting
 ***************************************************************************/
	
proc(2)=up1(xhat);
@ proc calculates a forecast value for each month if explanatory variables for y1 for 
	month t are given by row t of x1hat and and those for y2 for month t are given 
	by row t of y1 ~ xhat @
local y1out, y2out;
y1out = xhat*phihat';
if icont==1;
	y1out[.,4] = kcont;
elseif icont==2;
	y1out[.,4] = y1out[.,4] + 0.25;
endif;
y2out = (y1out ~ xhat)*dhat';
retp(y1out,y2out);
endp;

y1out = 0;     @ phantom entry to load next proc @
y2out = 0;
proc(0)=nexthor;
@ this proc takes global forecast input xq and output y1out, y2out and uses it to create
	a new global input matrix to add one month to the forecast horizon @
local it;
it = 1;
do until it > nuse;
	xq[it,2:cols(xq)] = readj(  (y1out[it,.] ~ y2out[it,.])' , xq[it,2:cols(xq)] , ny  );
it = it+1;
endo;
xq = zeros(1,cols(xq)) | xq[1:nuse-1,.];    @ this makes row t variables appropriate
		to form an ih-period-ahead forecast of target for month t @
{y1out,y2out} = up1(xq);
endp;

/*************************************************************************************
	form base case VAR forecasts and put in matrices
	fore1, fore2, etc. are each (capt x ny) matrices.  row tau, col. j of forek is the forecast for 
	the value that variable j will take on during month tau, where the forcast is based on
	information at tau - k.  Note in this convention that tau=1 corresponds to ob # nfest 
******************************************************************************************/

xq = xq0;

{y1out,y2out} = up1(xq);
fore1 = y1out~y2out;

hor = 12;      @ hor is the number of period-ahead forecasts calculated @
ih = 2;
do until ih > hor;
	call nexthor;
	field = 1;
	prec = 0;
	fmat = "%*.*lf";
	zz = ftos(ih,fmat,field,prec);
	qstring = "fore" $+ zz;
	junny = y1out~y2out;
	junny[1:ih,.]  = zeros(ih,ny);
	call varput(junny,qstring);
ih = ih+1;
endo;

/***********************************************************************
	print out results if desired
**********************************************************************/

/*
"===============================================";
"Forecast summary";
"Row t of matrix forej is forecast of vector of observations for month t, with forecast formed"
"at month t-j"  
"";"------------------------------------------------------------------------------------------";
"fore1";fore1;
ih = 2;
do until ih > hor;
	zz = ftos(ih,fmat,field,prec);
	qstring = "fore" $+ zz;
	"";"------------------------------------------------------------------------------------------";
	qstring;
	q = varget(qstring);
	q;
ih = ih+1;
endo;
*/

/*********************************************************************************************
	calculate usual VAR impulse-response function to check that code is correct 
 *********************************************************************************************/
varimp = zeros(hor,ny);
	@ row i, col j of varimp is response of variable j to shock to fed funds rate (j-1) periods
		previously @

xq = xq0;
icont = 2;
{y1out,y2out} = up1(xq);
icont = 0;
bump1 = y1out~y2out;
varimp[1,.] = meanc(bump1[2:nq,.] - fore1[2:nq,.])';
ih = 2;
do until ih > hor;
	call nexthor;
	field = 1;
	prec = 0;
	fmat = "%*.*lf";
	zz = ftos(ih,fmat,field,prec);
	qstring = "bump" $+ zz;
	junny = y1out~y2out;
	junny[1:ih,.]  = zeros(ih,ny) ;
	call varput(junny,qstring);
	qstring = "fore" $+ zz;
	q = varget(qstring);
	varimp[ih,.] = meanc(junny[ih+1:nq,.] - q[ih+1:nq,.])';
ih = ih+1;
endo;

"VAR impulse response function";
varimp = varimp/.25;
varimp;

"normalized for comparison with RATS";
varimp*.4467;

/*************************************************************************************
	Form VAR forecasts that use ACH simulations with target for month tau equal to 
	target for tau -1 and put in matrices.
	it1_1, it1_2, etc. are each (capt x ny) matrices.  row tau, col. j of it1_k is the forecast for 
	the value that variable j will take on during month tau, where the forcast is based on
	information at tau - k.  Note in this convention that tau = 1 corresponds to ob # nfest 
******************************************************************************************/

xq = xq0;
icont = 1;
kcont = lagtarmon[.,2];
{y1out,y2out} = up1(xq);
it1_1 = y1out~y2out;

ih = 2;
do until ih > hor;
	kcont = it1mon[.,ih];
	call nexthor;
	field = 1;
	prec = 0;
	fmat = "%*.*lf";
	zz = ftos(ih,fmat,field,prec);
	qstring = "it1_" $+ zz;
	junny = y1out~y2out;
	call varput(junny,qstring);
ih = ih+1;
endo;

/*************************************************************************************
	Form VAR forecasts that use ACH simulations with target for month tau equal to 
	target for tau -1 plus 25 basis points and put in matrices.
	it1and25_1, it1and25_2, etc. are each (capt x ny) matrices.  
	row tau, col. j of it1and25_k is the forecast for 
	the value that variable j will take on during month tau, where the forcast is based on
	information at tau - k.  Note in this convention that tau = 1 corresponds to ob # nfest 
******************************************************************************************/

it1and25imp = zeros(hor,ny);
xq = xq0;
icont = 1;
kcont = lagtarmon[.,2]+0.25;
{y1out,y2out} = up1(xq);
bump1 = y1out~y2out;
it1and25imp[1,.] = meanc(bump1 - it1_1)';

ih = 2;
do until ih > hor;
	kcont = it1and25mon[.,ih];
	call nexthor;
	field = 1;
	prec = 0;
	fmat = "%*.*lf";
	zz = ftos(ih,fmat,field,prec);
	qstring = "bump" $+ zz;
	junny = y1out~y2out;
	call varput(junny,qstring);
	qstring = "it1_" $+ zz;
	q = varget(qstring);
	it1and25imp[ih,.] = meanc(junny - q)';	
ih = ih+1;
endo;

"effect of raising target 25 basis points";
it1and25imp = it1and25imp/.25;
it1and25imp;



/*************************************************************************************
	Form VAR forecasts that use ACH simulations with target for month tau equal to 
	forecast of target formed at end of month tau - 1 and put in matrices.
	ihat_1, ihat_2, etc. are each (capt x ny) matrices.  
	row tau, col. j of ihat_k is the forecast for 
	the value that variable j will take on during month tau, where the forcast is based on
	information at tau - k.  Note in this convention that tau = 1 corresponds to ob # nfest 
******************************************************************************************/

ihatimp = zeros(hor,ny);
thresh = 0.05;    @ thresh is threshold for whether we count difference between i(tau-1)
				and ihat(tau) in calculating average impulse-response function @
xq = xq0;
icont = 1;
kcont = basemon[.,2];
wts = lagtarmon[.,2] - basemon[.,2];
wts = (1 ./ wts) .* (abs(wts) .> thresh);
"weights";
wts';
"";

junk = sumc( wts .* (lagtarmon[.,2] - basemon[.,2]) )';	
junk = junk / (sumc( wts ./= 0));
"check: following should be unity";;junk;
junk = lagtarmon[.,2] - it1_1[.,4];
junk = junk'*junk;
"check: following should be zero";;junk;

{y1out,y2out} = up1(xq);

junk = basemon[.,2] - y1out[.,4];
junk = junk'*junk;
"check: following should be zero";;junk;

bump1 = y1out~y2out;
ihatimp[1,.] = sumc( wts .* (it1_1 - bump1) )';	
ihatimp[1,.] = ihatimp[1,.] / (sumc( wts ./= 0));

ih = 2;
do until ih > hor;
	kcont = ihatmon[.,ih];
	call nexthor;
	field = 1;
	prec = 0;
	fmat = "%*.*lf";
	zz = ftos(ih,fmat,field,prec);
	qstring = "bump" $+ zz;
	junny = y1out~y2out;
	call varput(junny,qstring);
	qstring = "it1_" $+ zz;
	q = varget(qstring);
	ihatimp[ih,.] = sumc( wts .* (junny - q) )';	
	ihatimp[ih,.] = ihatimp[ih,.] / (sumc( wts ./= 0));
ih = ih+1;
endo;

"effect of target = forecast";
ihatimp;

output off;
output file=varimp.out reset;
varimp;
output off;

output file=i1_25imp.out reset;
it1and25imp;
output off;

output file=ihatimp.out reset;
ihatimp;
output off;


